searchData={"items":[{"type":"behaviour","title":"BatchServing","doc":"BatchServing encapsulates client and server work to perform batched requests.\n\nServings can be executed on the fly, without starting a server, but most\noften they are used to run servers that batch requests until a given size\nor timeout is reached.\n\nMore specifically, servings are a mechanism to apply a computation on a\n`BatchServing.Batch`, with hooks for preprocessing input from and postprocessing\noutput for the client. Thus we can think of an instance of `t:BatchServing.t/0`\n(a serving) as something that encapsulates batches of Nx computations.","ref":"BatchServing.html"},{"type":"behaviour","title":"Inline/serverless workflow - BatchServing","doc":"We can use `new/1` to create a serving that will return a JIT\nor AOT compiled function to execute on batches of tensors:\n\n\n    iex> serving = BatchServing.new(fn _opts -> fn a -> Enum.map(a.stack, &(&1 * &1)) end end)\n    iex> batch = BatchServing.Batch.stack([1, 2, 3, 4])\n    iex> BatchServing.run(serving, batch)\n    [1, 4, 9, 16]\n\nWhen defining a `Serving`, we can also customize how the data is\nbatched by using the `client_preprocessing` as well as the result by\nusing `client_postprocessing` hooks.\n\n    iex> serving = (\n    ...>   BatchServing.new(fn _opts -> fn a -> Enum.map(a.stack, &(&1 * &1)) end end)\n    ...>   |> BatchServing.client_preprocessing(fn input -> {input, :client_info} end)\n    ...>   |> BatchServing.client_postprocessing(&{&1, &2})\n    ...> )\n    iex> batch = BatchServing.Batch.stack([1, 2, 3, 4])\n    iex> BatchServing.run(serving, batch)\n    {{[1, 4, 9, 16],\n      :server_info},\n     :client_info}\n\nYou can see the results are a bit different now. First of all, notice that\nwe were able to run the serving passing a list of tensors. Our custom\n`client_preprocessing` function stacks those tensors into a batch of two\nentries and returns a tuple with a `BatchServing.Batch` struct and additional client\ninformation which we represent as the atom `:client_info`. The default\nclient preprocessing simply enforces a batch (or a stream of batches)\nwas given and returns no client information.\n\nThen the result is a triplet tuple, returned by the client\npostprocessing function, containing the result, the server information\n(which we will later learn how to customize), and the client information.\nFrom this, we can infer the default implementation of `client_postprocessing`\nsimply returns the result, discarding the server and client information.\n\nSo far, `Serving` has not given us much. It has simply encapsulated the\nexecution of a function. Its full power comes when we start running our own\n`Serving` process. That's when we will also learn why we have a `client_`\nprefix in some of the function names.","ref":"BatchServing.html#module-inline-serverless-workflow"},{"type":"behaviour","title":"Stateful/process workflow - BatchServing","doc":"`Serving` allows us to define an Elixir process to handle requests.\nThis process provides several features, such as batching up to a given\nsize or time, partitioning, and distribution over a group of nodes.\n\nTo do so, we need to start a `BatchServing` process with a serving inside\na supervision tree:\n\n    children = [\n      {BatchServing,\n       serving: BatchServing.new(fn _opts -> fn a -> Enum.map(IO.inspect(a.stack), &(&1 * &1)) end end),\n       name: MyServing,\n       batch_size: 10,\n       batch_timeout: 100}\n    ]\n\n    Supervisor.start_child(children, strategy: :one_for_one)\n\n> Note: in your actual application, you want to make sure\n> `Serving` comes early in your supervision tree, for example\n> before your web application endpoint or your data processing\n> pipelines, as those processes may end-up hitting BatchServing.\n\nNow you can send batched runs to said process:\n\n    iex> batch = BatchServing.Batch.stack([[1, 2, 3], [4, 5, 6]])\n    iex> BatchServing.batched_run(MyServing, batch)\n    [\n      [2, 4, 6],\n      [8, 10, 12]\n    ]\n\nIn the example, we pushed a batch of 2 and eventually got a reply.\nThe process will wait for requests from other processes, for up to\n100 milliseconds or until it gets 10 entries. Then it merges all\nbatches together and once the result is computed, it slices and\ndistributes those responses to each caller.\n\nIf there is any `client_preprocessing` function, it will be executed\nbefore the batch is sent to the server. If there is any `client_postprocessing`\nfunction, it will be executed after getting the response from the\nserver.","ref":"BatchServing.html#module-stateful-process-workflow"},{"type":"behaviour","title":"Partitioning - BatchServing","doc":"You can start several partitions under the same serving by passing\n`partitions: true` when starting the serving. The number of partitions\nwill be determined according  your compiler and for which host it is\ncompiling.\n\nFor example, when creating the serving, you may pass the following\n`defn_options`:\n\n    BatchServing.new(computation, compiler: EXLA, client: :cuda)\n\nNow when booting up the serving:\n\n    children = [\n      {BatchServing,\n       serving: serving,\n       name: MyServing,\n       batch_size: 10,\n       batch_timeout: 100,\n       partitions: true}\n    ]","ref":"BatchServing.html#module-partitioning"},{"type":"behaviour","title":"Distribution - BatchServing","doc":"All `Serving`s are distributed by default. If the current machine\ndoes not have an instance of `Serving` running, `batched_run/3` will\nautomatically look for one in the cluster. The nodes do not need to run\nthe same code and applications. It is only required that they run the\nsame `Nx` version.\n\nThe load balancing between servings is done randomly, however, the number\nof partitions are considered if the `partitions: true` option is also given.\n\nThe servings are dispatched using Erlang Distribution. You can use\n`Node.connect/1` to manually connect nodes. In a production setup, this is\noften done with the help of libraries like [`libcluster`](https://github.com/bitwalker/libcluster).","ref":"BatchServing.html#module-distribution"},{"type":"behaviour","title":"Advanced notes - BatchServing","doc":"","ref":"BatchServing.html#module-advanced-notes"},{"type":"behaviour","title":"Module-based serving - BatchServing","doc":"In the examples so far, we have been using the default version of\n`Serving`, which executes the given function for each batch.\n\nHowever, we can also use `new/2` to start a module-based version of\n`Serving` which gives us more control over both inline and process\nworkflows. A simple module implementation of a `Serving` could look\nlike this:\n\n    defmodule MyServing do\n      @behaviour BatchServing\n\n      @impl true\n      def init(_inline_or_process, :unused_arg, [defn_options]) do\n        {:ok, fn a -> Enum.map(IO.inspect(a.stack), &(&1 * &1)) end}\n      end\n\n      @impl true\n      def handle_batch(batch, 0, function) do\n        {:execute, fn -> {function.(batch), :server_info} end, function}\n      end\n    end\n\nIt has two functions. The first, `c:init/3`, receives the type of serving\n(`:inline` or `:process`) and the serving argument.\n\nThe second function is called `c:handle_batch/3`. This function\nreceives a `BatchServing.Batch` and returns a function to execute.\nThe function itself must return a two element-tuple: the batched\nresults and some server information. The server information can\nbe any value and we set it to the atom `:server_info`.\n\nNow let's give it a try by defining a serving with our module and\nthen running it on a batch:\n\n    iex> serving = BatchServing.new(MyServing, :unused_arg)\n    iex> batch = BatchServing.Batch.stack([[1, 2, 3]])\n    iex> BatchServing.run(serving, batch)\n    [[1, 4, 9]]\n\nFrom here on, you use `start_link/1` to start this serving in your\nsupervision and even customize `client_preprocessing/1` and\n`client_postprocessing/1` callbacks to this serving, as seen in the\nprevious sections.\n\nNote in our implementation above assumes it won't run partitioned.\nIn partitioned mode, `c:init/3` may receive multiple `defn_options`\nas the third argument and `c:handle_batch/3` may receive another partition\nbesides 0.","ref":"BatchServing.html#module-module-based-serving"},{"type":"behaviour","title":"Streaming - BatchServing","doc":"`Serving` allows both inputs and outputs to be streamed.\n\nIn order to stream inputs, you only need to return a stream of `BatchServing.Batch`\nfrom the `client_preprocessing` callback. BatchServing will automatically take\ncare of streaming the inputs in, regardless if using `run/2` or `batched_run/2`.\nIt is recommended that the streaming batches have the same size as `batch_size`,\nto avoid triggering `batch_timeout` on every iteration (except for the last one\nwhich may be incomplete).\n\nTo stream outputs, you must invoke `streaming/2` with any additional\nstreaming configuration. When this is invoked, the `client_postprocessing`\nwill receive a stream which you can further manipulate lazily using the\nfunctions in the `Stream` module. `streaming/2` also allows you to configure\nhooks and stream values directly from `Nx.Defn` hooks. However, when hook\nstreaming is enabled, certain capabilities are removed: you cannot stream\ninputs nor have batches larger than the configured `batch_size`.\n\nYou can enable both input and output streaming at once.","ref":"BatchServing.html#module-streaming"},{"type":"behaviour","title":"Batch keys - BatchServing","doc":"Sometimes it may be necessary to execute different functions under the\nsame serving.\n\nBatch keys provide a mechanism to accumulate different batches, based on\ntheir key, which execute independently. As an example, we will do a\nserving which performs different operations based on the batch key,\nbut it could also be used to perform the same operation for different\ntemplates:\n\n    iex> serving = BatchServing.new(fn\n    ...>   :double, opts -> &Enum.map(&1.stack, fn v -> v * 2 end)\n    ...>   :half, opts -> &Enum.map(&1.stack, fn v -> v / 2 end)\n    ...> end)\n    iex> double_batch = BatchServing.Batch.concatenate([0, 1, 2, 3, 4, 5, 6, 7, 8, 9]) |> BatchServing.Batch.key(:double)\n    iex> BatchServing.run(serving, double_batch)\n    [0, 2, 4, 6, 8, 10, 12, 14, 16, 18]\n    iex> half_batch = BatchServing.Batch.concatenate([0, 1, 2, 3, 4, 5, 6, 7, 8, 9]) |> BatchServing.Batch.key(:half)\n    iex> BatchServing.run(serving, half_batch)\n    [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]\n\nWhen using a process-based serving, you must specify the supported\n`:batch_keys` when the process is started. The batch keys will be\navailable inside the `defn_options` passed as the third argument of\nthe `c:init/3` callback. The batch keys will also be verified\nwhen the batch is returned from the client-preprocessing callback.","ref":"BatchServing.html#module-batch-keys"},{"type":"function","title":"BatchServing.batch_size/2","doc":"Sets the batch size for this serving.\n\nThis batch size is used to split batches given to both `run/2` and\n`batched_run/2`, enforcing that the batch size never goes over a limit.\nIf you only want to batch within the serving process, you must set\n`:batch_size` via `process_options/2` (or on `start_link/1`).\n\n> #### Why batch on `run/2`? {: .info}\n>\n> By default, `run/2` does not place a limit on its input size. It always\n> processes inputs directly within the current process. On the other hand,\n> `batched_run/2` always sends your input to a separate process, which\n> will batch and execute the serving only once the batch is full or a\n> timeout has elapsed.\n>\n> However, in some situations, an input given to `run/2` needs to be\n> broken into several batches. If we were to very large batches to our\n> computation, the computation could require too much memory. In such\n> cases, setting a batch size even on `run/2` is beneficial, because\n> BatchServing takes care of splitting a large batch into smaller ones\n> that do not exceed the `batch_size` value.","ref":"BatchServing.html#batch_size/2"},{"type":"function","title":"BatchServing.batched_run/3","doc":"Runs the given `input` on the serving process given by `name`.\n\n`name` is either an atom representing a local or distributed\nserving process. First it will attempt to dispatch locally, then it\nfalls back to the distributed serving. You may specify\n`{:local, name}` to force a local lookup or `{:distributed, name}`\nto force a distributed one.\n\nThe `client_preprocessing` callback will be invoked on the `input`\nwhich is then sent to the server. The server will batch requests\nand send a response either when the batch is full or on timeout.\nThen `client_postprocessing` is invoked on the response. See the\nmodule documentation for more information. In the distributed case,\nthe callbacks are invoked in the distributed node, but still outside of\nthe serving process.\n\nNote that you cannot batch an `input` larger than the configured\n`:batch_size` in the server.","ref":"BatchServing.html#batched_run/3"},{"type":"function","title":"Distributed mode - BatchServing.batched_run/3","doc":"To run in distributed mode, the nodes do not need to run the same\ncode and applications. It is only required that they run the\nsame `Nx` version.\n\nIf the current node is running a serving given by `name` locally\nand `{:distributed, name}` is used, the request will use the same\ndistribution mechanisms instead of being handled locally, which\nis useful for testing locally without a need to spawn nodes.\n\nThis function receives an optional `distributed_preprocessing` callback as\nthird argument for preprocessing the input for distributed requests. When\nusing libraries like EXLA or Torchx, the tensor is often allocated in memory\ninside a third-party library so it may be necessary to either transfer or copy\nthe tensor to the binary backend before sending it to another node.\nThis can be done by passing either `Nx.backend_transfer/1` or `Nx.backend_copy/1`\nas third argument:\n\n    BatchServing.batched_run(MyDistributedServing, input, &Nx.backend_copy/1)\n\nUse `backend_transfer/1` if you know the input will no longer be used.\n\nSimilarly, the serving has a `distributed_postprocessing` callback which can do\nequivalent before sending the reply to the caller.","ref":"BatchServing.html#batched_run/3-distributed-mode"},{"type":"function","title":"BatchServing.client_postprocessing/2","doc":"Sets the client postprocessing function.\n\nThe client postprocessing receives a tuple with the\n`{output, metadata}` or a stream as first argument.\nThe second argument is always the additional information\nreturned by the client preprocessing.\n\nThe default implementation returns either the output or\nthe stream.","ref":"BatchServing.html#client_postprocessing/2"},{"type":"function","title":"BatchServing.client_preprocessing/2","doc":"Sets the client preprocessing function.\n\nThe default implementation expects a `BatchServing.Batch` or a stream of\nBatchServing.Batch to be given as input and return them as is.","ref":"BatchServing.html#client_preprocessing/2"},{"type":"function","title":"BatchServing.create_serving_process_group_spec/0","doc":"","ref":"BatchServing.html#create_serving_process_group_spec/0"},{"type":"function","title":"BatchServing.default_options/0","doc":"","ref":"BatchServing.html#default_options/0"},{"type":"function","title":"BatchServing.defn_options/2","doc":"Sets the defn options of this serving.\n\nThese are the options supported by `Serving.default_options/1`.","ref":"BatchServing.html#defn_options/2"},{"type":"function","title":"BatchServing.distributed_postprocessing/2","doc":"Sets the distributed postprocessing function.\n\nThe default implementation is `Function.identity/1`.","ref":"BatchServing.html#distributed_postprocessing/2"},{"type":"callback","title":"BatchServing.handle_batch/3","doc":"Receives a batch, a partition, and returns a function to execute the batch.\n\nIn case of serving processes, the function is executed is an\nseparate process.","ref":"BatchServing.html#c:handle_batch/3"},{"type":"callback","title":"BatchServing.init/3","doc":"The callback used to initialize the serving.\n\nThe first argument reveals if the serving is executed inline,\nsuch as by calling `run/2`, by started with the process.\nThe second argument is the serving argument given to `new/2`.\nThe third argument option is a list of compiler options to be\nused to compile each partition the serving will run.\n\nIt must return `{:ok, state}`, where the `state` can be any term.","ref":"BatchServing.html#c:init/3"},{"type":"function","title":"BatchServing.new/2","doc":"Creates a new function serving.\n\nIt expects a single- or double-arity function. If a single-arity\nfunction is given, it receives the compiler options and must\nreturn a one-arity function.\n\nIf a double-arity function is given, it receives the batch\nkey as first argument and the compiler options as second argument.\nIt must return a one-arity function. The batch keys can be given on\n`start_link/1`.\n\nThe function will be called with the arguments returned by the\n`client_preprocessing` callback.","ref":"BatchServing.html#new/2"},{"type":"function","title":"BatchServing.new/3","doc":"Creates a new module-based serving.\n\nIt expects a module and an argument that is given to its `init`\ncallback.\n\nA third optional argument called `defn_options` are additional\ncompiler options which will be given to the module. Those options\nwill be merged into `Serving.default_options/0`.","ref":"BatchServing.html#new/3"},{"type":"function","title":"BatchServing.process_options/2","doc":"Sets the process options of this serving.\n\nThese are the same options as supported on `start_link/1`,\nexcept `:name` and `:serving` itself.","ref":"BatchServing.html#process_options/2"},{"type":"function","title":"BatchServing.run/2","doc":"Runs `serving` with the given `input` inline with the current process.\n\nThe `serving` is executed immediately, without waiting or batching inputs\nfrom other processes. If a `batch_size/2` is specified, then the input may\nbe split or padded, but they are still executed immediately inline.","ref":"BatchServing.html#run/2"},{"type":"function","title":"BatchServing.start_link/1","doc":"Starts a `Serving` process to batch requests to a given serving.","ref":"BatchServing.html#start_link/1"},{"type":"function","title":"Options - BatchServing.start_link/1","doc":"All options, except `:name` and `:serving`, can also be set via\n`process_options/2`.\n\n  * `:name` - an atom with the name of the process\n\n  * `:serving` - a `Serving` struct with the serving configuration\n\n  * `:batch_keys` - all available batch keys. Batch keys allows Serving\n    to accumulate different batches with different properties. Defaults to\n    `[:default]`\n\n  * `:batch_size` - the maximum batch size. A default value can be set with\n    `batch_size/2`, which applies to both `run/2` and `batched_run/2`.\n    Setting this option only affects `batched_run/2` and it defaults to `1`\n    if none is set.\n\n  * `:batch_timeout` - the maximum time to wait, in milliseconds,\n    before executing the batch (defaults to `100`ms)\n\n  * `:partitions` - when `true`, starts several partitions under this serving.\n    The number of partitions will be determined according to your compiler\n    and for which host it is compiling. See the module docs for more information\n\n  * `:shutdown` - the maximum time for the serving to shutdown. This will\n    block until the existing computation finishes (defaults to `30_000`ms)\n\n  * `:hibernate_after` and `:spawn_opt` - configure the underlying serving\n    workers (see `GenServer.start_link/3`)","ref":"BatchServing.html#start_link/1-options"},{"type":"function","title":"BatchServing.streaming/2","doc":"Configure the serving to stream its results.\n\nOnce `run/2` or `batched_run/2` are invoked, it will then\nreturn a stream. The stream must be consumed in the same\nprocess that calls `run/2` or `batched_run/2`.\n\nBatches will be streamed as they arrive. You may also opt-in\nto stream `Nx.Defn` hooks.","ref":"BatchServing.html#streaming/2"},{"type":"function","title":"Options - BatchServing.streaming/2","doc":"* `:hooks` - a list of hook names that will become streaming events","ref":"BatchServing.html#streaming/2-options"},{"type":"function","title":"Implementation details - BatchServing.streaming/2","doc":"","ref":"BatchServing.html#streaming/2-implementation-details"},{"type":"function","title":"Client postprocessing - BatchServing.streaming/2","doc":"Once streaming is enabled, the client postprocessing callback\nwill receive a stream which will emit events for each hook\nin the shape of:\n\n    {hook_name, term()}\n\nThe stream will also receive events in the shape of\n`{:batch, output, metadata}` as batches are processed by the\nserving. The client postprocessing is often expected to call\n`Stream.transform/3` to process those events into something\nusable by callers.\n\nIf the `:hooks` option is given, only a single `:batch` event\nis emitted, at the end, as detailed next.","ref":"BatchServing.html#streaming/2-client-postprocessing"},{"type":"function","title":"Batch limits - BatchServing.streaming/2","doc":"If you are streaming hooks, the serving server can no longer break\nbatch and you are unable to push a payload bigger than `:batch_size`.\nFor example, imagine you have a `batch_size` of 3 and you push three\nbatches of two elements (AA, BB, and CC). Without hooks, the batches\nwill be consumed as:\n\n    AAB -> BCC\n\nWith streaming, we can't break the batch `BB`, as above, so we will\nconsistently pad with zeroes:\n\n    AA0 -> BB0 -> CC0\n\nIn practice, this should not be a major problem, as you should\ngenerally avoid having a batch size that is not a multiple of the\nmost common batches.","ref":"BatchServing.html#streaming/2-batch-limits"},{"type":"type","title":"BatchServing.client_info/0","doc":"","ref":"BatchServing.html#t:client_info/0"},{"type":"type","title":"BatchServing.client_postprocessing/0","doc":"","ref":"BatchServing.html#t:client_postprocessing/0"},{"type":"type","title":"BatchServing.client_preprocessing/0","doc":"","ref":"BatchServing.html#t:client_preprocessing/0"},{"type":"type","title":"BatchServing.distributed_postprocessing/0","doc":"","ref":"BatchServing.html#t:distributed_postprocessing/0"},{"type":"type","title":"BatchServing.distributed_preprocessing/0","doc":"","ref":"BatchServing.html#t:distributed_preprocessing/0"},{"type":"type","title":"BatchServing.metadata/0","doc":"","ref":"BatchServing.html#t:metadata/0"},{"type":"type","title":"BatchServing.t/0","doc":"","ref":"BatchServing.html#t:t/0"},{"type":"module","title":"BatchServing.Batch","doc":"Creates a batch of values.\n\nA batch is lazily traversed, concatenated, and padded upon `defn` invocation.","ref":"BatchServing.Batch.html"},{"type":"function","title":"BatchServing.Batch.__struct__/0","doc":"A BatchServing.Batch struct.\n\nThe `:size` field is public.","ref":"BatchServing.Batch.html#__struct__/0"},{"type":"function","title":"BatchServing.Batch.concatenate/2","doc":"Concatenates the given entries to the batch.\n\nEntries are concatenated based on their first axis.\nIf the first axis has multiple entries, each entry\nis added to the size of the batch.\n\nYou can either concatenate to an existing batch\nor skip the batch argument to create a new batch.\n\nSee `stack/2` if you want to stack entries instead\nof concatenating them.","ref":"BatchServing.Batch.html#concatenate/2"},{"type":"function","title":"Examples - BatchServing.Batch.concatenate/2","doc":"If no batch is given, one is automatically created:\n\n    iex> batch = BatchServing.Batch.concatenate([1, 2, 3])\n    iex> batch.stack\n    [1, 2, 3]\n\nBut you can also concatenate to existing batches:\n\n    iex> batch = BatchServing.Batch.concatenate(BatchServing.Batch.stack([1]), ([2]))\n    iex> batch = BatchServing.Batch.concatenate(batch, [3, 4])\n    iex> batch.stack\n    [1, 2, 3, 4]\n\nIf the first axis has multiple entries, each entry counts\ntowards the size of the batch:\n\n    iex> batch = BatchServing.Batch.concatenate(BatchServing.Batch.stack([1, 2]), [3, 4, 5])\n    iex> batch.size\n    5\n    iex> batch.stack\n    [1, 2, 3, 4, 5]","ref":"BatchServing.Batch.html#concatenate/2-examples"},{"type":"function","title":"BatchServing.Batch.key/2","doc":"Sets the batch key for the given batch.","ref":"BatchServing.Batch.html#key/2"},{"type":"function","title":"BatchServing.Batch.merge/1","doc":"Merges a list of batches.\n\nSee `merge/2`.","ref":"BatchServing.Batch.html#merge/1"},{"type":"function","title":"BatchServing.Batch.merge/2","doc":"Merges two batches.\n\nThe values on the left will appear before the values on the right.\n\nThe size and padding of both batches are summed. The padding still\napplies only at the end of batch.\n\nIt will raise if the batch templates are incompatible.","ref":"BatchServing.Batch.html#merge/2"},{"type":"function","title":"Examples - BatchServing.Batch.merge/2","doc":"iex> batch1 = BatchServing.Batch.stack([1, 2, 3])\n    iex> batch2 = BatchServing.Batch.concatenate(BatchServing.Batch.stack([4, 5]), [6, 7, 8])\n    iex> batch = BatchServing.Batch.merge(batch1, batch2)\n    iex> batch.size\n    8\n    iex> batch.stack\n    [1, 2, 3, 4, 5, 6, 7, 8]","ref":"BatchServing.Batch.html#merge/2-examples"},{"type":"function","title":"BatchServing.Batch.new/0","doc":"Returns a new empty batch.","ref":"BatchServing.Batch.html#new/0"},{"type":"function","title":"BatchServing.Batch.split/2","doc":"Splits a batch in two, where the first one has at most `n` elements.\n\nIf there is any padding and the batch is not full, the amount of padding\nnecessary will be moved to the first batch and the remaining stays in the\nsecond batch.","ref":"BatchServing.Batch.html#split/2"},{"type":"function","title":"Examples - BatchServing.Batch.split/2","doc":"iex> batch = BatchServing.Batch.concatenate(BatchServing.Batch.stack([1, 2]), [3, 4, 5])\niex> {left, right} = BatchServing.Batch.split(batch, 3)\niex> left.stack\n[1, 2, 3]\niex> right.stack\n[4, 5]","ref":"BatchServing.Batch.html#split/2-examples"},{"type":"function","title":"BatchServing.Batch.stack/2","doc":"Stacks the given entries to the batch.\n\nEach entry counts exactly as a single entry.\nYou can either stack to an existing batch\nor skip the batch argument to create a new batch.\n\nSee `concatenate/2` if you want to concatenate entries\ninstead of stacking them.","ref":"BatchServing.Batch.html#stack/2"},{"type":"function","title":"Examples - BatchServing.Batch.stack/2","doc":"If no batch is given, one is automatically created:\n\n    iex> batch = BatchServing.Batch.stack([1, 2, 3])\n    iex> batch.size\n    3\n    iex> batch.stack\n    [1, 2, 3]\n\nBut you can also stack an existing batch:\n\n    iex> batch = BatchServing.Batch.stack([1, 2])\n    iex> batch = BatchServing.Batch.stack(batch, [3, 4])\n    iex> batch.size\n    4\n    iex> batch.stack\n    [1, 2, 3, 4]","ref":"BatchServing.Batch.html#stack/2-examples"},{"type":"type","title":"BatchServing.Batch.t/0","doc":"","ref":"BatchServing.Batch.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}